import 'dart:async';
import 'package:flutter/material.dart';
import '../flutter_flicker.dart';
import '../flutter_flicker_controls.dart';

///Some sort of comment
class FlickerConfiguration {
  static void set(
      {Widget? unknownScreen,
      Widget? errorScreen,
      Widget? waitingScreen,
      List<RouteInfo>? routes,
      RouteInfo? dashboardRoute,
      List<Menu>? menus,
      FlickerColors? colors,
      Completer<void>? isAppInitialised,
      FlickerDisplayOptions? displayOptions}) {
    if (unknownScreen != null)
      FlickerConfiguration.unknownScreen = unknownScreen;
    if (errorScreen != null) FlickerConfiguration.errorScreen = errorScreen;
    if (waitingScreen != null)
      FlickerConfiguration.waitingScreen = waitingScreen;
    if (routes != null) FlickerConfiguration.routes = routes;
    if (dashboardRoute != null)
      FlickerConfiguration.dashboardRoute = dashboardRoute;
    if (menus != null) FlickerConfiguration.menus = menus;
    if (colors != null) FlickerConfiguration.colors = colors;
    FlickerConfiguration.displayOptions =
        displayOptions ?? FlickerDisplayOptions();
    if (isAppInitialised != null)
      FlickerConfiguration.isAppInitialised = isAppInitialised;
  }

  static Widget? unknownScreen;
  static Widget? errorScreen;
  static Widget? waitingScreen;
  static Widget? getWaitingScreen() {
    if (waitingScreen != null) return waitingScreen;
    return Scaffold(
      body: Center(
        child: CircularProgressIndicator(),
      ),
    );
  }

  static Widget? getUnknownScreen() {
    if (unknownScreen != null) return unknownScreen;
    return Scaffold(
      body: Center(
        child: Column(
          mainAxisAlignment: MainAxisAlignment.center,
          crossAxisAlignment: CrossAxisAlignment.center,
          children: [
            Text(
              'The route you have found yourself at does not exist. So sorry :(',
            ),
            SizedBox(
              height: 10,
            ),
            FlickerElevatedButton(
              onPressed: () => currentRouteSingleton.navigateHome(),
              child: Text('Ok'),
            ),
          ],
        ),
      ),
    );
  }

  static Widget? getErrorScreen() {
    if (errorScreen != null) return errorScreen;
    return Scaffold(
      body: Center(
        child: Text('${BaseLogic.mostRecentError}'),
      ),
    );
  }

  static late List<RouteInfo> routes;
  static RouteInfo? dashboardRoute;
  static late List<Menu> menus;
  static late FlickerColors colors;
  static late FlickerDisplayOptions displayOptions;
  static Completer<void>? isAppInitialised;
}

class FlickerDisplayOptions {
  late bool showHintText;
  late bool uppercaseLabelAndHint;
  FlickerDisplayOptions(
      {this.showHintText = true, this.uppercaseLabelAndHint = true});
}
