import 'dart:math';

import 'package:flutter/material.dart';

class FlickerFlexibleColumn extends StatelessWidget {
  final double minWidthPerChild;
  final List<Widget>? children;
  final int maxChildrenPerRow;
  final double maxWidthPerChild;
  final EdgeInsetsGeometry padding;

  final CrossAxisAlignment crossAxisAlignment;

  const FlickerFlexibleColumn({
    Key? key,
    this.children,
    this.minWidthPerChild = 300,
    this.maxWidthPerChild = 2000,
    this.maxChildrenPerRow = 999,
    this.crossAxisAlignment = CrossAxisAlignment.start,
    this.padding = const EdgeInsets.only(
      right: 8.0,
    ),
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: padding,
      child: LayoutBuilder(
        builder: (BuildContext context, BoxConstraints constraints) {
          return Column(
            mainAxisSize: MainAxisSize.min,
            crossAxisAlignment: crossAxisAlignment,
            children: getRows(context, constraints.maxWidth),
          );
        },
      ),
    );
  }

  List<Row> getRows(BuildContext context, double maxWidth) {
    print(maxWidth);
    print(minWidthPerChild);
    var numberPerRow = max((maxWidth / minWidthPerChild).floor(), 1);
    if (numberPerRow > maxChildrenPerRow) numberPerRow = maxChildrenPerRow;
    print(numberPerRow);
    var rows = <Row>[];
    var countForRow = 0;
    var currentRowChildren = <Widget>[];
    for (var child in children!) {
      currentRowChildren.add(
        Expanded(
          child: Container(
            child: child,
            margin: EdgeInsets.only(
              top: 8.0,
              right: 8.0,
            ),
          ),
        ),
      );
      countForRow++;
      if (countForRow == numberPerRow) {
        rows.add(
          Row(
            crossAxisAlignment: CrossAxisAlignment.start,
            mainAxisSize: MainAxisSize.max,
            children: currentRowChildren,
          ),
        );
        currentRowChildren = [];
        countForRow = 0;
      }
    }

    if (currentRowChildren.isNotEmpty) {
      while (countForRow < numberPerRow) {
        currentRowChildren.add(
          Expanded(child: Container()),
        );
        countForRow++;
      }
      rows.add(
        Row(
          crossAxisAlignment: CrossAxisAlignment.start,
          mainAxisSize: MainAxisSize.max,
          children: currentRowChildren,
        ),
      );
    }

    return rows;
  }
}
