import 'package:flutter/material.dart';
import 'package:flutter_flicker/configuration/configuration.dart';
import 'decorator_utils.dart';

class FlickerDropDown<T> extends StatelessWidget {
  FlickerDropDown({
    Key? key,
    required this.labelText,
    this.uppercaseLabelAndHint = true,
    this.initialSelection,
    this.items,
    required this.onChanged,
    this.validator,
    this.optionTitles,
    this.readOnly = false,
    this.hintText,
  }) : super(key: key);

  final List<T>? items;
  final T? initialSelection;
  final String labelText;
  final bool? uppercaseLabelAndHint;
  final ValueChanged<T?> onChanged;
  final Function? validator;
  final List<String>? optionTitles;
  final bool readOnly;
  final String? hintText;

  @override
  Widget build(BuildContext context) {
    return Container(
      width: 250,
      alignment: Alignment.center,
      child: DropdownButtonFormField<T>(
        items: buildItemList(),
        value: initialSelection,
        onChanged: readOnly ? null : onChanged,
        validator: validator as String? Function(T?)?,
        isExpanded: true,
        style: Theme.of(context).textTheme.subtitle1?.copyWith(
              color: FlickerConfiguration.colors.input,
            ),
        decoration: InputDecoration(
          labelText: DecoratorUtils.buildLabelText(
            labelText,
            uppercase: uppercaseLabelAndHint,
          ),
          hintText: DecoratorUtils.buildHintText(
            null,
            labelText: labelText,
            uppercase: uppercaseLabelAndHint,
            readOnly: readOnly,
          ),
          labelStyle:
              Theme.of(context).inputDecorationTheme.labelStyle?.copyWith(
                    color: readOnly
                        ? Colors.grey.shade700
                        : FlickerConfiguration.colors.inputLabel,
                  ),
          hintStyle: Theme.of(context).inputDecorationTheme.hintStyle?.copyWith(
                color: FlickerConfiguration.colors.hintText,
              ),
          enabledBorder: Theme.of(context)
              .inputDecorationTheme
              .enabledBorder
              ?.copyWith(
                borderSide: Theme.of(context)
                    .inputDecorationTheme
                    .enabledBorder
                    ?.borderSide
                    .copyWith(
                      color:
                          FlickerConfiguration.colors.inputControlEnabledBorder,
                    ),
              ),
          focusedBorder: Theme.of(context)
              .inputDecorationTheme
              .focusedBorder
              ?.copyWith(
                borderSide: Theme.of(context)
                    .inputDecorationTheme
                    .focusedBorder
                    ?.borderSide
                    .copyWith(
                      color:
                          FlickerConfiguration.colors.inputControlFocusedBorder,
                    ),
              ),
          disabledBorder:
              Theme.of(context).inputDecorationTheme.disabledBorder?.copyWith(
                    borderSide: Theme.of(context)
                        .inputDecorationTheme
                        .disabledBorder
                        ?.borderSide
                        .copyWith(
                          color: FlickerConfiguration
                              .colors.inputControlDisabledBorder,
                        ),
                  ),
          border: Theme.of(context).inputDecorationTheme.border?.copyWith(
                borderSide: Theme.of(context)
                    .inputDecorationTheme
                    .border
                    ?.borderSide
                    .copyWith(
                      color: FlickerConfiguration.colors.inputControlBorder,
                    ),
              ),
        ),
      ),
    );
  }

  List<DropdownMenuItem<T>> buildItemList() {
    var itemList = <DropdownMenuItem<T>>[];
    for (var i = 0; i < items!.length; i++) {
      itemList.add(DropdownMenuItem<T>(
        child: Text(
            optionTitles == null ? items![i].toString() : optionTitles![i]),
        value: items![i],
      ));
    }
    return itemList;
  }
}
