import 'package:flutter/material.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:flutter_riverpod/flutter_riverpod.dart';
import 'package:flutter_flicker/controls/form_input/date_input.dart';
import 'package:flutter_flicker/configuration/configuration.dart';
import 'package:flutter_flicker/theme/colors.dart';

// todo: <extract shared test setup (this was copied from dfp)
typedef TestWidgetBuilder = Widget Function();

Widget createAppToWrapTestWidget(TestWidgetBuilder builder) => ProviderScope(
  child: MaterialApp(
    home: Scaffold(
      body: builder(),
    ),
  )
);

void setScreenSize(WidgetTester tester, {double w = 1600, double h = 1280}) {
  tester.binding.window.physicalSizeTestValue = Size(w, h);
  tester.binding.window.devicePixelRatioTestValue = 1.0;
}

// todo: extract/>

void  main() {
  FlickerConfiguration.set(
    colors: FlickerColors(),
  );

  final isRunningInUtcPlus2 = DateTime.now().timeZoneOffset.inMinutes == 120;

  group('FlickerDateInput', () {
    testWidgets('loads correctly with default (no) parameters', (WidgetTester tester) async {
      setScreenSize(tester);
      await tester.pumpWidget(createAppToWrapTestWidget(() => FlickerDateInput()));
    });

    group('change initialDateValue to local time', () {
      testWidgets('...20T20 -> ...20... (still 20)', (WidgetTester tester) async {
        setScreenSize(tester);

        final dateInUtc = DateTime.parse('1969-07-20T20:18:04Z');
        await tester.pumpWidget(createAppToWrapTestWidget(() => FlickerDateInput(
          initialDateValue: dateInUtc,
        )));

        expect(find.text('1969-07-20'), findsOneWidget, reason: 'still on 20th');
      });

      testWidgets('...15T22 -> ...16T00', (WidgetTester tester) async {
        setScreenSize(tester);

        final dateAtEndOfUtcDay = DateTime.parse('2021-11-15T22:00:00Z');
        await tester.pumpWidget(createAppToWrapTestWidget(() => FlickerDateInput(
          initialDateValue: dateAtEndOfUtcDay,
        )));

        expect(find.text('2021-11-16'), findsOneWidget, reason: '22:00 utc is 00:00 next day local');
      });
    }, skip: isRunningInUtcPlus2 ? null : 'tests are written specifically for utc+2 (example SAST) timezone');
  });
}
